/**************************************************************************
*  Copyright (c) 2016 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  27.12.2016  mifi  First Version for the Altera DE1-SoC board.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdint.h>

#include "c5soc.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/* HPS_LED is on GPIO[53] */
#define HPS_LED_MASK 0x01000000

#define LED_ON()     { GPIO1->DR |= HPS_LED_MASK;  }
#define LED_OFF()    { GPIO1->DR &= ~HPS_LED_MASK; } 

typedef void (*irq_fnt)(void);


/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

irq_fnt interrupt_handlers[MAX_IRQ_COUNT];

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static volatile uint32_t dSystemTick = 0;

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  SysTick_Handler                                                      */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void SysTick_Handler (void)
{
   dSystemTick++;   
} /* SysTick_Handler */

/*************************************************************************/
/*  TimeWait                                                             */
/*                                                                       */
/*  Loop for a specified number of ticks, will not release the CPU.      */
/*                                                                       */
/*  In    : TimeoutMs                                                    */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void TimeWait (uint32_t dTicks)
{
   uint32_t dTimeEnd;
   
   dTimeEnd = dSystemTick + dTicks;
   
   while (dSystemTick <= dTimeEnd)
   {
      __asm volatile ("nop");
   }
      
} /* TimeWait */

/***************************************************************************/
/*  GICInit                                                                */
/*                                                                         */
/*  Global Interrupt Controller initializing                               */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void GICInit (void)
{
   uint32_t Index;
   
   /* 
    * Disable global CPU interrupt 
    */
   __asm volatile ("CPSID i");

   /* 
    * Disable GIC distributor 
    */
   GIC_DIST->ICDDCR = 0;
      
   /* 
   * Clear all interrupt vectors 
   */   
   for(Index=0; Index<MAX_IRQ_COUNT; Index++)
   {
      interrupt_handlers[Index] = 0;
   }
   
   /* 
    * Disable all interrupt bits and set processor target 
    */
   for (Index=0; Index<(MAX_IRQ_COUNT / 32); Index++)
   {
      GIC_DIST->ICDICER[Index] = 0xFFFFFFFF;
      
      if (Index >= 32)
      {
         /* Select "CPU interface 0" for the interrupts*/
         GIC_DIST->ICDIPTR[Index/4] |= 1 << ((Index % 4) * 8);
      } 
   }
   
   /* 
    * Set "Interrupt Priority Mask"
    * See GIC 1.0, section 4.4.2
    */
   GIC->ICCPMR = 0xFF;
   
   /* 
    * Set "Priority grouping"
    * See GIC 1.0, section 3.3.3
    */   
   GIC->ICCBPR = 7;  /* 7 = no preemption */
   
   /* 
    * Enables the signalling of interrupts to the target processors
    */
   GIC->ICCICR = 1;

   /* 
    * Enable global CPU interrupt 
    */
   __asm volatile ("CPSIE i");

   /* 
    * Enable GIC distributor 
    */
   GIC_DIST->ICDDCR = 1;
   
} /* GICInit */

/***************************************************************************/
/*  LEDInit                                                                */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void LEDInit (void)
{
   GPIO1->DR  &= ~HPS_LED_MASK; 
   GPIO1->DDR |= HPS_LED_MASK;

} /* LEDInit */

/***************************************************************************/
/*  SysTickInit                                                            */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void SysTickInit (void)
{
   uint32_t vector   = PrivateTimer_IRQn;
   uint32_t priority = 0;
   
   /* Disable global CPU interrupt */ 
   __asm volatile ("CPSID i");   
   
   priority &= __GIC_PRIO_MASK;
   priority  = priority << __GIC_PRIO_SHIFT;
   
   interrupt_handlers[vector] = SysTick_Handler;

   /* See GIC 1.0, section 4.3.10 */
   GIC_DIST->ICDIPR[vector/4] |= (priority << (((vector % 4) * 8))); 

   /* Enable */
   /* See GIC 1.0, section 4.3.5 */
   GIC_DIST->ICDISER[vector/32] |= (1 << (vector % 32));

   /* Setup Timer */   
   PRIVATE_TIMER->Load    = (MPU_PERIPH_CLK_FREQ / 1000) - 1;
   PRIVATE_TIMER->Control = 0x7; 
     
   /* Enable global CPU interrupt */
   __asm volatile ("CPSIE i");
} /* SysTickInit */

/***************************************************************************/
/*  HardwareInit                                                           */
/*                                                                         */
/*  In    : none                                                           */
/*  Out   : none                                                           */
/*  Return: none                                                           */
/***************************************************************************/
static void HardwareInit (void)
{
   /* Init the Global Interrupt Controller */
   GICInit();

   /* Init LED */
   LEDInit();
   
   /* Init SysTick */
   SysTickInit();
   
} /* HardwareInit */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
int main (void)
{
   /*
    * Before the application can be started, the hardware of the target
    * must be initialized. The CPU is still initialized by the "Bootloader"
    * and CrossWorks for ARM startup. But the timer tick  must be init too.
    * Even the GPIO can be init here.
    */
   HardwareInit();
   
   while (1)
   {
      LED_ON();
      TimeWait(500);
      LED_OFF();
      TimeWait(500);
   }

   /*
    * This return here make no sense.
    * But to prevent the compiler warning:
    * "return type of 'main' is not 'int'
    * we use an int as return :-)
    */ 
   return(0);
} /* main */

/*** EOF ***/
